#version 330

#define PI 3.1415926535897932384626433832795

uniform sampler2D 	framebufferObjectTexture;
uniform sampler2D 	framebufferDepthTexture;


uniform int			sampleCount;
uniform float		maximalCoC;

//not used in the simple model:
uniform float		focalLength;
uniform float		sigma;

uniform vec2 		direction; //= vec2(0.0, 1.0) || vec2(1.0, 0.0)

uniform float    	focusDistance; //== depth of the ball
uniform float		linearBlurChange;

uniform float	  	zNearPlane;
uniform float 		zFarPlane;
uniform vec2		aspectWidthHeight;


in 	  vec2  		fragmentTexCoords;



layout(location = 0 ) out vec4 fragColor;


float linstep(float low, float high, float v){
    return clamp( (v - low)/(high - low), 0.0, 1.0);
}



float getLinearZBufferVal( vec2 coords){
	float znotlinear = texture( framebufferDepthTexture, coords );
	return (zNearPlane * znotlinear) / (zFarPlane - znotlinear * (zFarPlane - zNearPlane) );
}


void main(void){
	//get the depth on this pixel:
	float depth = getLinearZBufferVal( fragmentTexCoords );
	//float focalLength = linstep( zNearPlane, zFarPlane, focalLength);

	//calc the aperture:
	//float aperture = linearBlurChange * focusDistance;

	//the magnification:
	//float magnification = linstep( zNearPlane, zFarPlane, focalLength) / (focusDistance - focalLength);
	//float magnification = focalLength / abs(focalLength);

	//CoC in world space:
	//float Cz = aperture * ( abs( depth - focusDistance )/depth );

	//in image space:
	//float cz = Cz * magnification; 

	float ddiff = abs( depth - focusDistance );
	float cz = linearBlurChange * ddiff * ddiff;// -> simple model
	cz = clamp( cz, 0.0, maximalCoC );

	float blursteps = cz / sampleCount * 0.5; //0.5 because 2 directions


	//Incremental computation of the gaussian:
	//http://http.developer.nvidia.com/GPUGems3/gpugems3_ch40.html
	//float delta = 1.0/sampleCount;
	vec3 g;
	g.x = 1.0 / (sqrt(2.0 * PI) * sigma); 
	//g.y = exp(-0.5 * delta * delta / (sigma * sigma));  
	g.y = exp( -0.5 / (sigma * sigma) );
	g.z = g.y * g.y;  

	
	//problems with alpha < 1 areas... the diminish
	fragColor = texture( framebufferObjectTexture, fragmentTexCoords );
	fragColor.rgb *= g.x;
	
	float coeffsum = g.x;

	g.xy *= g.yz;


	for( int i = 1; i < sampleCount; ++i ){
		vec2 dcoord = blursteps * i * direction * aspectWidthHeight;
		
		fragColor.rgb += texture( framebufferObjectTexture, fragmentTexCoords + dcoord ).rgb * g.x;// * use1;
		fragColor.rgb += texture( framebufferObjectTexture, fragmentTexCoords - dcoord ).rgb * g.x;// * use2;
		
		//coeffsum += 2.0*g.x;
		g.xy *= g.yz;
	}

	//fragColor /= (coeffsum); //minimal different from 1... so ok, better to save the multiplications

	fragColor.a = 1.0; //because of this is the final image anyway it's ok.. else i would need to change the blending, use premultiplied alpha 

	//cz = cz * 10.0;
	//fragColor = vec4( cz, cz, cz, 1.0 );


}




