#version 330


uniform sampler2D framebufferObjectTexture; //the framebuffer rendered in a texture
uniform sampler2D framebufferDepthTexture;


const int motionBlurImageCount = 1; //same as in config.ini, except 0.. then 1
uniform sampler2D motionBlurImages[motionBlurImageCount];
uniform int newestMotionBlurImage;


uniform float	 zNearPlane; //depth debugging
uniform float    zFarPlane;

in vec2 		fragmentTexCoords;

layout(location = 0 ) out vec4 fragColor;



//real z value
float getRealZBufferVal(){
	float znotlinear = texture( framebufferDepthTexture, fragmentTexCoords );
	float znorm = 2.0 * znotlinear - 1.0;
	return 2.0 * zNearPlane * zFarPlane / (zFarPlane + zNearPlane - znorm * ( zFarPlane - zNearPlane ) );
}

float getLinearZBufferVal(){
	float znotlinear = texture( framebufferDepthTexture, fragmentTexCoords );
	return (zNearPlane * znotlinear) / (zFarPlane - znotlinear * (zFarPlane - zNearPlane) );
}



void main(void)
{	
	fragColor = texture( framebufferObjectTexture, fragmentTexCoords ); 

	//Motion Blur:
	if( motionBlurImageCount > 1 ){
		float counter = motionBlurImageCount;
		float multisum = motionBlurImageCount + 1;
		fragColor *= counter;

		for( int i = newestMotionBlurImage; i > 0; --i ){
			fragColor += 2.0 * counter * texture( motionBlurImages[i], fragmentTexCoords );
			multisum += counter * 2.0;
			--counter;
		}

		for( int i = motionBlurImageCount - 1; i > newestMotionBlurImage; --i ){
			fragColor += 2.0 * counter * texture( motionBlurImages[i], fragmentTexCoords );
			multisum += counter * 2.0;
			--counter;
		}
		fragColor /= multisum;
	}

	
	//fragColor = clamp( fragColor, 0.0, 1.0 );

	//float depth = getLinearZBufferVal();
	//fragColor.rgb = vec3(depth, depth, depth);
	//if( depth == 0.0 ) fragColor = vec4( 1.0, 1.0, 0.0, 1.0 );

	//if( fragColor.r != 0.0 ) fragColor = vec4( 1.0, 0.0, 0.0, 1.0 );
}

